package de.hft.stuttgart.citydoctor2.gui;

import java.io.File;
import java.io.IOException;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.hft.stuttgart.citydoctor2.utils.Localization;
import javafx.application.Platform;
import javafx.event.Event;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.CheckBox;
import javafx.scene.control.Label;
import javafx.scene.control.ProgressBar;
import javafx.scene.control.TextField;
import javafx.scene.control.TextFormatter;
import javafx.scene.control.TitledPane;
import javafx.scene.image.Image;
import javafx.scene.input.KeyCode;
import javafx.scene.input.KeyEvent;
import javafx.scene.layout.VBox;
import javafx.stage.FileChooser;
import javafx.stage.FileChooser.ExtensionFilter;
import javafx.stage.Modality;
import javafx.stage.Stage;
import javafx.stage.Window;

public class OpenFileDialog {

	private static final Logger logger = LogManager.getLogger(OpenFileDialog.class);

	private Stage stage;

	@FXML
	private Button loadBtn;

	@FXML
	private Button cancelBtn;

	@FXML
	private Button selectBtn;

	@FXML
	private TextField precisionField;

	@FXML
	private TextField pathField;

	@FXML
	private ProgressBar progress;

	@FXML
	private CheckBox useValidationBox;

	@FXML
	private Label fileLabel;

	@FXML
	private TitledPane settingsPane;

	@FXML
	private Label roundingPlacesLabel;

	@FXML
	private Label xmlValidationLabel;

	@FXML
	private Label lowMemoryLabel;

	@FXML
	private CheckBox lowMemoryBox;

	private CityDoctorController controller;
	private ExceptionDialog exDialog;
	private FileChooser fc;

	public OpenFileDialog(Window parent, CityDoctorController controller) throws IOException {
		FXMLLoader loader = new FXMLLoader(OpenFileDialog.class.getResource("OpenFileDialog.fxml"));
		loader.setController(this);
		VBox box = loader.load();
		this.controller = controller;
		stage = new Stage();
		stage.getIcons().add(new Image(MainWindow.class.getResourceAsStream("icons/CityDoctor-Logo-rot_klein.jpg")));
		stage.setScene(new Scene(box));
		stage.initOwner(parent);
		stage.initModality(Modality.APPLICATION_MODAL);
		stage.setTitle("Open File");
		stage.getScene().addEventFilter(KeyEvent.KEY_PRESSED, (KeyEvent event) -> {
			if (event.getCode() == KeyCode.ESCAPE) {
				stage.close();
			}
		});
	}

	public void initialize() {
		cancelBtn.setOnAction(ae -> stage.close());
		setupPrecisionField();
		setupLoadButton();
		setupSelectButton();
		applyLanguageToControls();
	}

	private void applyLanguageToControls() {
		fileLabel.setText(Localization.getText("OpenFileDialog.fileLabel"));
		selectBtn.setText(Localization.getText("OpenFileDialog.selectBtn"));
		loadBtn.setText(Localization.getText("OpenFileDialog.loadBtn"));
		settingsPane.setText(Localization.getText("OpenFileDialog.settingsPane"));
		roundingPlacesLabel.setText(Localization.getText("OpenFileDialog.roundingPlacesLabel"));
		xmlValidationLabel.setText(Localization.getText("OpenFileDialog.xmlValidationLabel"));
		cancelBtn.setText(Localization.getText("OpenFileDialog.cancelBtn"));
		lowMemoryLabel.setText(Localization.getText("OpenFileDialog.lowMemoryLabel"));
	}

	private void setupSelectButton() {
		selectBtn.setOnAction(ae -> {
			if (fc == null) {
				fc = new FileChooser();
				fc.setTitle(Localization.getText("OpenFileDialog.select"));
				fc.getExtensionFilters().add(new ExtensionFilter("GML/XML", "*.gml", "*.xml"));
				fc.getExtensionFilters().add(new ExtensionFilter("ZIP", "*.zip"));
				fc.getExtensionFilters().add(new ExtensionFilter(Localization.getText("MainWindow.all"), "*.*"));
			}
			File dir = new File(Settings.get(Settings.LAST_OPEN_FOLDER, ""));
			if (dir.exists() && dir.isDirectory()) {
				fc.setInitialDirectory(dir);
			} else {
				String userDir = System.getProperty("user.dir");
				Settings.set(Settings.LAST_OPEN_FOLDER, userDir);
				fc.setInitialDirectory(new File(userDir));
			}
			File f = fc.showOpenDialog(stage);
			if (f != null) {
				Settings.set(Settings.LAST_OPEN_FOLDER, f.getParent());
				pathField.setText(f.getAbsolutePath());
			}
		});
	}

	private void setupLoadButton() {
		loadBtn.setOnAction(ae -> {
			int numberOfRoundingPlaces = Integer.parseInt(precisionField.getText());
			boolean useValidation = useValidationBox.isSelected();
			boolean lowMemory = lowMemoryBox.isSelected();
			String path = pathField.getText();
			cancelBtn.setDisable(true);
			loadBtn.setDisable(true);
			pathField.setDisable(true);
			selectBtn.setDisable(true);
			stage.setOnCloseRequest(Event::consume);
			Thread t = new Thread(() -> {
				try {
					controller.loadCityGml(path, numberOfRoundingPlaces, progress::setProgress, useValidation,
							lowMemory);
					Platform.runLater(() -> stage.close());
				} catch (Exception e) {
					if (logger.isErrorEnabled()) {
						logger.error(Localization.getText("OpenFileDialog.loadFailed"), e);
					}
					Platform.runLater(() -> {
						if (exDialog == null) {
							exDialog = new ExceptionDialog();
						}
						exDialog.show(e);
					});
				} finally {
					selectBtn.setDisable(false);
					pathField.setDisable(false);
					cancelBtn.setDisable(false);
					loadBtn.setDisable(false);
					stage.setOnCloseRequest(null);
				}
			});
			t.start();
		});
	}

	private void setupPrecisionField() {
		TextFormatter<String> formatter = new TextFormatter<>(change -> {
			if (!change.isContentChange()) {
				return change;
			}

			String text = change.getControlNewText();
			try {
				Integer.parseInt(text);
				return change;
			} catch (NumberFormatException e) {
				return null;
			}
		});
		precisionField.setTextFormatter(formatter);
	}

	public void show() {
		Platform.runLater(() -> {
			progress.setProgress(0d);
			stage.showAndWait();
		});
	}

}
